/*
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_0.txt.                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: SRA OSS, Inc. Japan <info@sraoss.co.jp>                     |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#ifndef PHP_TEXTCONV_H
#define PHP_TEXTCONV_H

#ifdef  __cplusplus
extern "C" {
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <php.h>

#ifdef HAVE_TEXTCONV
#define PHP_TEXTCONV_VERSION "1.1.1"


#include <php_ini.h>
#include <SAPI.h>
#include <ext/standard/info.h>
#include <Zend/zend_extensions.h>
#ifdef  __cplusplus
} // extern "C" 
#endif
#include <textconv/textconv.h>
#include <readarchive.h>
#ifdef  __cplusplus
extern "C" {
#endif

extern zend_module_entry textconv_module_entry;
#define phpext_textconv_ptr &textconv_module_entry

#ifdef PHP_WIN32
#define PHP_TEXTCONV_API __declspec(dllexport)
#else
#define PHP_TEXTCONV_API
#endif

PHP_MINIT_FUNCTION(textconv);
PHP_MSHUTDOWN_FUNCTION(textconv);
PHP_RINIT_FUNCTION(textconv);
PHP_RSHUTDOWN_FUNCTION(textconv);
PHP_MINFO_FUNCTION(textconv);

#ifdef ZTS
#include "TSRM.h"
#endif

#define FREE_RESOURCE(resource) zend_list_delete(Z_LVAL_P(resource))

#define PROP_GET_LONG(name)    Z_LVAL_P(zend_read_property(_this_ce, _this_zval, #name, strlen(#name), 1 TSRMLS_CC))
#define PROP_SET_LONG(name, l) zend_update_property_long(_this_ce, _this_zval, #name, strlen(#name), l TSRMLS_CC)

#define PROP_GET_DOUBLE(name)    Z_DVAL_P(zend_read_property(_this_ce, _this_zval, #name, strlen(#name), 1 TSRMLS_CC))
#define PROP_SET_DOUBLE(name, d) zend_update_property_double(_this_ce, _this_zval, #name, strlen(#name), d TSRMLS_CC)

#define PROP_GET_STRING(name)    Z_STRVAL_P(zend_read_property(_this_ce, _this_zval, #name, strlen(#name), 1 TSRMLS_CC))
#define PROP_GET_STRLEN(name)    Z_STRLEN_P(zend_read_property(_this_ce, _this_zval, #name, strlen(#name), 1 TSRMLS_CC))
#define PROP_SET_STRING(name, s) zend_update_property_string(_this_ce, _this_zval, #name, strlen(#name), s TSRMLS_CC)
#define PROP_SET_STRINGL(name, s, l) zend_update_property_stringl(_this_ce, _this_zval, #name, strlen(#name), s, l TSRMLS_CC)


PHP_FUNCTION(textconv_convert_file);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_convert_file_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 4)
  ZEND_ARG_INFO(0, filename)
  ZEND_ARG_INFO(0, mime_type)
  ZEND_ARG_INFO(0, flags)
  ZEND_ARG_INFO(1, text)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
static unsigned char textconv_convert_file_arg_info[] = {4, BYREF_NONE, BYREF_NONE, BYREF_NONE, BYREF_FORCE};
#endif

PHP_FUNCTION(textconv_convert_data);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_convert_data_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 5)
  ZEND_ARG_INFO(0, data)
  ZEND_ARG_INFO(0, len)
  ZEND_ARG_INFO(0, mime_type)
  ZEND_ARG_INFO(0, flags)
  ZEND_ARG_INFO(1, text)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
static unsigned char textconv_convert_data_arg_info[] = {5, BYREF_NONE, BYREF_NONE, BYREF_NONE, BYREF_NONE, BYREF_FORCE};
#endif

PHP_FUNCTION(textconv_get_mimetype_from_ext);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_get_mimetype_from_ext_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
  ZEND_ARG_INFO(0, filename)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_get_mimetype_from_ext_arg_info NULL
#endif

PHP_FUNCTION(textconv_get_mimetype_from_data);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_get_mimetype_from_data_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 2)
  ZEND_ARG_INFO(0, data)
  ZEND_ARG_INFO(0, len)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_get_mimetype_from_data_arg_info NULL
#endif

PHP_FUNCTION(textconv_set_error_mode);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_set_error_mode_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
  ZEND_ARG_INFO(0, error_mode)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_set_error_mode_arg_info NULL
#endif

PHP_FUNCTION(textconv_set_debug_mode);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_set_debug_mode_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
  ZEND_ARG_INFO(0, debug_mode)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_set_debug_mode_arg_info NULL
#endif

PHP_FUNCTION(textconv_get_error_mode);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_get_error_mode_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_get_error_mode_arg_info NULL
#endif

PHP_FUNCTION(textconv_get_debug_mode);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_get_debug_mode_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_get_debug_mode_arg_info NULL
#endif

PHP_FUNCTION(textconv_get_version_string);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(textconv_get_version_string_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define textconv_get_version_string_arg_info NULL
#endif

PHP_FUNCTION(archive_file_open);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_open_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 2)
  ZEND_ARG_INFO(0, file)
  ZEND_ARG_INFO(0, mime_type)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define archive_file_open_arg_info NULL
#endif

PHP_FUNCTION(archive_file_open_data);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_open_data_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 3)
  ZEND_ARG_INFO(0, data)
  ZEND_ARG_INFO(0, len)
  ZEND_ARG_INFO(0, mime_type)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define archive_file_open_data_arg_info NULL
#endif

PHP_FUNCTION(archive_file_open_full);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_open_full_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 3)
  ZEND_ARG_INFO(0, file)
  ZEND_ARG_INFO(0, mime_type)
  ZEND_ARG_INFO(1, error)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
static unsigned char archive_file_open_full_arg_info[] = {3, BYREF_NONE, BYREF_NONE, BYREF_FORCE};
#endif

PHP_FUNCTION(archive_file_open_data_full);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_open_data_full_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 4)
  ZEND_ARG_INFO(0, data)
  ZEND_ARG_INFO(0, len)
  ZEND_ARG_INFO(0, mime_type)
  ZEND_ARG_INFO(1, error)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
static unsigned char archive_file_open_data_full_arg_info[] = {4, BYREF_NONE, BYREF_NONE, BYREF_NONE, BYREF_FORCE};
#endif

PHP_FUNCTION(archive_file_get_next_filename);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_get_next_filename_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 2)
  ZEND_ARG_INFO(0, afile)
  ZEND_ARG_INFO(1, filename)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
static unsigned char archive_file_get_next_filename_arg_info[] = {2, BYREF_NONE, BYREF_FORCE};
#endif

PHP_FUNCTION(archive_file_get_next_data);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_get_next_data_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 3)
  ZEND_ARG_INFO(0, afile)
  ZEND_ARG_INFO(1, data)
  ZEND_ARG_INFO(1, len)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
static unsigned char archive_file_get_next_data_arg_info[] = {3, BYREF_NONE, BYREF_FORCE, BYREF_FORCE};
#endif

PHP_FUNCTION(archive_file_close);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_close_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
  ZEND_ARG_INFO(0, afile)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define archive_file_close_arg_info NULL
#endif

PHP_FUNCTION(archive_file_set_error_mode);
#if (PHP_MAJOR_VERSION >= 5)
ZEND_BEGIN_ARG_INFO_EX(archive_file_set_error_mode_arg_info, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
  ZEND_ARG_INFO(0, error_mode)
ZEND_END_ARG_INFO()
#else /* PHP 4.x */
#define archive_file_set_error_mode_arg_info NULL
#endif

#ifdef  __cplusplus
} // extern "C" 
#endif

#endif /* PHP_HAVE_TEXTCONV */

#endif /* PHP_TEXTCONV_H */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
