/*
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_0.txt.                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: SRA OSS, Inc. Japan <info@sraoss.co.jp>                     |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#include "php_textconv.h"

#if HAVE_TEXTCONV

/* {{{ Resource destructors */
int le_archivefile;
void archivefile_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	ArchiveFile * resource = (ArchiveFile *)(rsrc->ptr);

	do {
	} while (0);
}

/* }}} */

/* {{{ textconv_functions[] */
#if PHP_VERSION_ID <= 50299
  function_entry textconv_functions[] = {
#else
  zend_function_entry textconv_functions[] = {
#endif
	PHP_FE(textconv_convert_file, textconv_convert_file_arg_info)
	PHP_FE(textconv_convert_data, textconv_convert_data_arg_info)
	PHP_FE(textconv_get_mimetype_from_ext, textconv_get_mimetype_from_ext_arg_info)
	PHP_FE(textconv_get_mimetype_from_data, textconv_get_mimetype_from_data_arg_info)
	PHP_FE(textconv_set_error_mode, textconv_set_error_mode_arg_info)
	PHP_FE(textconv_set_debug_mode, textconv_set_debug_mode_arg_info)
	PHP_FE(textconv_get_error_mode, textconv_get_error_mode_arg_info)
	PHP_FE(textconv_get_debug_mode, textconv_get_debug_mode_arg_info)
	PHP_FE(textconv_get_version_string, textconv_get_version_string_arg_info)
	PHP_FE(archive_file_open   , archive_file_open_arg_info)
	PHP_FE(archive_file_open_data, archive_file_open_data_arg_info)
	PHP_FE(archive_file_open_full, archive_file_open_full_arg_info)
	PHP_FE(archive_file_open_data_full, archive_file_open_data_full_arg_info)
	PHP_FE(archive_file_get_next_filename, archive_file_get_next_filename_arg_info)
	PHP_FE(archive_file_get_next_data, archive_file_get_next_data_arg_info)
	PHP_FE(archive_file_close  , archive_file_close_arg_info)
	PHP_FE(archive_file_set_error_mode, archive_file_set_error_mode_arg_info)
	{ NULL, NULL, NULL }
};
/* }}} */


/* {{{ textconv_module_entry
 */
zend_module_entry textconv_module_entry = {
	STANDARD_MODULE_HEADER,
	"textconv",
	textconv_functions,
	PHP_MINIT(textconv),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN(textconv), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT(textconv),     /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN(textconv), /* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO(textconv),
	PHP_TEXTCONV_VERSION, 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_TEXTCONV
ZEND_GET_MODULE(textconv)
#endif


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(textconv)
{
	REGISTER_LONG_CONSTANT("TC_OK", TC_OK, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_ERROR", TC_ERROR, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_NOT_EXIST", TC_NOT_EXIST, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_UNKNOWN_FORMAT", TC_UNKNOWN_FORMAT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_IO_ERROR", TC_IO_ERROR, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_INVALID_CONTENT", TC_INVALID_CONTENT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_INVALID_ARG", TC_INVALID_ARG, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_ENCRYPTED", TC_ENCRYPTED, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_AUTODETECT_FILETYPE_FROM_DATA", TC_AUTODETECT_FILETYPE_FROM_DATA, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_GET_PROPERTY_AND_TEXT", TC_GET_PROPERTY_AND_TEXT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("TC_GET_PROPERTY_ONLY", TC_GET_PROPERTY_ONLY, CONST_PERSISTENT | CONST_CS);
	le_archivefile = zend_register_list_destructors_ex(archivefile_dtor, 
						   NULL, "archivefile", module_number);

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(textconv)
{

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION(textconv)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION(textconv)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(textconv)
{
	php_printf("libTextConv PHP interface\n");
	php_info_print_table_start();
	php_info_print_table_row(2, "Version",PHP_TEXTCONV_VERSION " (stable)");
	php_info_print_table_row(2, "Released", "2012-11-01");
	php_info_print_table_row(2, "CVS Revision", "$Id: textconv.c,v 1.6 2019/03/06 07:07:54 y-hiroyu Exp $");
	php_info_print_table_row(2, "Authors", "SRA OSS, Inc. Japan 'info@sraoss.co.jp' (lead)\n");
	php_info_print_table_end();
	/* add your stuff here */

}
/* }}} */


/* {{{ proto int textconv_convert_file(string filename, string mime_type, int flags, mixed &text)
  Converts file to plain text. */
PHP_FUNCTION(textconv_convert_file)
{

	const char * filename = NULL;
	int filename_len = 0;
	const char * mime_type = NULL;
	int mime_type_len = 0;
	long flags = 0;
	zval * text = NULL;
	int retval = 0;
	char *ret_text = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sslz", &filename, &filename_len, &mime_type, &mime_type_len, &flags, &text) == FAILURE) {
		return;
	}

	if (mime_type_len == 0)
		mime_type = NULL;
	retval = textconv_convert_file(filename, mime_type, flags, &ret_text);

	if (retval != 0) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "textconv_convert_file: returned error: %d", retval);
		RETURN_LONG(retval);
	}

	zval_dtor(text);
	if (ret_text) {
		ZVAL_STRING(text, ret_text, 1);
		textconv_free(ret_text);
	} else {
		ZVAL_EMPTY_STRING(text);
	}

	RETURN_LONG(retval);
}
/* }}} textconv_convert_file */


/* {{{ proto int textconv_convert_data(string data, int len, string mime_type, int flags, mixed &text)
  Converts data in memory to plain text. */
PHP_FUNCTION(textconv_convert_data)
{

	const char * data = NULL;
	int data_len = 0;
	long len = 0;
	const char * mime_type = NULL;
	int mime_type_len = 0;
	long flags = 0;
	zval * text = NULL;
	int retval = 0;
	char *ret_text = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "slslz", &data, &data_len, &len, &mime_type, &mime_type_len, &flags, &text) == FAILURE) {
		return;
	}

	if (mime_type_len == 0)
		mime_type = NULL;
	retval = textconv_convert_data((const unsigned char *)data, len, mime_type, flags, &ret_text);

	if (retval != 0) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "textconv_convert_data: returned error: %d", retval);
		RETURN_LONG(retval);
	}

	zval_dtor(text);
	if (ret_text) {
		ZVAL_STRING(text, ret_text, 1);
		textconv_free(ret_text);
	} else {
		ZVAL_EMPTY_STRING(text);
	}

	RETURN_LONG(retval);
}
/* }}} textconv_convert_data */


/* {{{ proto string textconv_get_mimetype_from_ext(string filename)
  Returns MIME type for given filename extension. */
PHP_FUNCTION(textconv_get_mimetype_from_ext)
{

	const char * filename = NULL;
	int filename_len = 0;
	const char *mime_type = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
		return;
	}

	mime_type = textconv_get_mimetype_from_ext(filename);
	if (mime_type) {
		RETURN_STRING((char *)mime_type, 1);
	} else {
		RETURN_NULL();
	}
}
/* }}} textconv_get_mimetype_from_ext */


/* {{{ proto string textconv_get_mimetype_from_data(string data, int len)
  Returns MIME type for given data. */
PHP_FUNCTION(textconv_get_mimetype_from_data)
{

	const char * data = NULL;
	int data_len = 0;
	long len = 0;
	const char *mime_type = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &data, &data_len, &len) == FAILURE) {
		return;
	}

	mime_type = textconv_get_mimetype_from_data((const unsigned char *)data, len);
	if (mime_type) {
		RETURN_STRING((char *)mime_type, 1);
	} else {
		RETURN_NULL();
	}
}
/* }}} textconv_get_mimetype_from_data */


/* {{{ proto void textconv_set_error_mode(bool error_mode)
  Changes error mode of libTextConv. */
PHP_FUNCTION(textconv_set_error_mode)
{

	zend_bool error_mode = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &error_mode) == FAILURE) {
		return;
	}

	textconv_set_error_mode(error_mode ? 1 : 0);
}
/* }}} textconv_set_error_mode */


/* {{{ proto void textconv_set_debug_mode(bool debug_mode)
  Changes debug mode of libTextConv. */
PHP_FUNCTION(textconv_set_debug_mode)
{

	zend_bool debug_mode = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &debug_mode) == FAILURE) {
		return;
	}

	textconv_set_debug_mode(debug_mode ? 1 : 0);
}
/* }}} textconv_set_debug_mode */


/* {{{ proto bool textconv_get_error_mode(void)
  Returns error mode of libTextConv. */
PHP_FUNCTION(textconv_get_error_mode)
{

	if (ZEND_NUM_ARGS()>0)  {
		WRONG_PARAM_COUNT;
	}

	if (textconv_get_error_mode()) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}
/* }}} textconv_get_error_mode */


/* {{{ proto bool textconv_get_debug_mode(void)
  Returns debug mode of libTextConv. */
PHP_FUNCTION(textconv_get_debug_mode)
{

	if (ZEND_NUM_ARGS()>0)  {
		WRONG_PARAM_COUNT;
	}

	if (textconv_get_debug_mode()) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}
/* }}} textconv_get_debug_mode */


/* {{{ proto string textconv_get_version_string(void)
  Returns version string of libTextConv. */
PHP_FUNCTION(textconv_get_version_string)
{
	const char *ver = NULL;

	if (ZEND_NUM_ARGS()>0)  {
		WRONG_PARAM_COUNT;
	}

	ver = textconv_get_version_string();
	if (ver == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "textconv_get_version_string: returned NULL");
		RETURN_NULL();
	}

	RETURN_STRING((char *)ver, 1);
}
/* }}} textconv_get_version_string */


/* {{{ proto resource archivefile archive_file_open(string file, string mime_type)
  Returns archivefile resource for given archive file. */
PHP_FUNCTION(archive_file_open)
{
	ArchiveFile * return_res;
	long return_res_id = -1;

	const char * file = NULL;
	int file_len = 0;
	const char * mime_type = NULL;
	int mime_type_len = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &file, &file_len, &mime_type, &mime_type_len) == FAILURE) {
		return;
	}

	if (mime_type_len == 0)
		mime_type = NULL;
	return_res = archive_file_open(file, mime_type);
	if (return_res == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_open: returned NULL");
		RETURN_NULL();
	}

	return_res_id = ZEND_REGISTER_RESOURCE(return_value, return_res, le_archivefile);
}
/* }}} archive_file_open */


/* {{{ proto resource archivefile archive_file_open_data(string data, int len, string mime_type)
  Returns archivefile resource for given archive data. */
PHP_FUNCTION(archive_file_open_data)
{
	ArchiveFile * return_res;
	long return_res_id = -1;

	const char * data = NULL;
	int data_len = 0;
	long len = 0;
	const char * mime_type = NULL;
	int mime_type_len = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sls", &data, &data_len, &len, &mime_type, &mime_type_len) == FAILURE) {
		return;
	}

	if (mime_type_len == 0)
		mime_type = NULL;
	return_res = archive_file_open_data((const unsigned char *)data, len, mime_type);
	if (return_res == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_open_data: returned NULL");
		RETURN_NULL();
	}

	return_res_id = ZEND_REGISTER_RESOURCE(return_value, return_res, le_archivefile);
}
/* }}} archive_file_open_data */


/* {{{ proto resource archivefile archive_file_open_full(string file, string mime_type, mixed &error)
  Returns archivefile resource with an error code for given archive file. */
PHP_FUNCTION(archive_file_open_full)
{
	ArchiveFile * return_res;
	long return_res_id = -1;

	const char * file = NULL;
	int file_len = 0;
	const char * mime_type = NULL;
	int mime_type_len = 0;
	zval * error = NULL;
	int ret_error = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ssz", &file, &file_len, &mime_type, &mime_type_len, &error) == FAILURE) {
		return;
	}

	if (mime_type_len == 0)
		mime_type = NULL;
	return_res = archive_file_open_full(file, mime_type, &ret_error);

	ZVAL_LONG(error, ret_error);

	if (return_res == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_open_full: returned NULL");
		RETURN_NULL();
	}

	return_res_id = ZEND_REGISTER_RESOURCE(return_value, return_res, le_archivefile);
}
/* }}} archive_file_open_full */


/* {{{ proto resource archivefile archive_file_open_data_full(string data, int len, string mime_type, mixed &error)
  Returns archivefile resource with an error code for given archive data. */
PHP_FUNCTION(archive_file_open_data_full)
{
	ArchiveFile * return_res;
	long return_res_id = -1;

	const char * data = NULL;
	int data_len = 0;
	long len = 0;
	const char * mime_type = NULL;
	int mime_type_len = 0;
	zval * error = NULL;
	int ret_error = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "slsz", &data, &data_len, &len, &mime_type, &mime_type_len, &error) == FAILURE) {
		return;
	}

	if (mime_type_len == 0)
		mime_type = NULL;
	return_res = archive_file_open_data_full((const unsigned char *)data, len, mime_type, &ret_error);

	ZVAL_LONG(error, ret_error);

	if (return_res == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_open_data_full: returned NULL");
		RETURN_NULL();
	}

	return_res_id = ZEND_REGISTER_RESOURCE(return_value, return_res, le_archivefile);
}
/* }}} archive_file_open_data_full */


/* {{{ proto int archive_file_get_next_filename(resource archivefile afile, mixed &filename)
  Returns next file name from given archive file. */
PHP_FUNCTION(archive_file_get_next_filename)
{
	zval * afile_res = NULL;
	int afile_resid = -1;
	ArchiveFile * afile;

	zval * filename = NULL;
	int retval = 0;
	char *ret_filename = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rz", &afile_res, &filename) == FAILURE) {
		return;
	}
	ZEND_FETCH_RESOURCE(afile, ArchiveFile *, &afile_res, afile_resid, "archivefile", le_archivefile);

	retval = archive_file_get_next_filename(afile, &ret_filename);
	
	if (retval != 0) {
		if (retval != -5) {
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_get_next_filename: returned error: %d", retval);
		}
		RETURN_LONG(retval);
	}

	zval_dtor(filename);
	if (ret_filename) {
		ZVAL_STRING(filename, ret_filename, 1);
		archive_file_mem_free(ret_filename);
	} else {
		ZVAL_EMPTY_STRING(filename);
	}

	RETURN_LONG(retval);
}
/* }}} archive_file_get_next_filename */


/* {{{ proto int archive_file_get_next_data(resource archivefile afile, mixed &data, mixed &len)
  Returns next data from given archive file. */
PHP_FUNCTION(archive_file_get_next_data)
{
	zval * afile_res = NULL;
	int afile_resid = -1;
	ArchiveFile * afile;

	zval * data = NULL;
	zval * len = NULL;
	int retval = 0;
	size_t ret_len = 0;
	unsigned char *ret_data = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rzz", &afile_res, &data, &len) == FAILURE) {
		return;
	}
	ZEND_FETCH_RESOURCE(afile, ArchiveFile *, &afile_res, afile_resid, "archivefile", le_archivefile);

	retval = archive_file_get_next_data(afile, &ret_data, &ret_len);

	if (retval != 0) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_get_next_data: returned error: %d", retval);
		RETURN_LONG(retval);
	}

	zval_dtor(data);
	if (ret_data) {
		ZVAL_STRINGL(data, ret_data, ret_len, 1);
		archive_file_mem_free(ret_data);
	} else {
		ZVAL_EMPTY_STRING(data);
	}

	ZVAL_LONG(len, ret_len);

	RETURN_LONG(retval);
}
/* }}} archive_file_get_next_data */


/* {{{ proto int archive_file_close(resource archivefile afile)
  Closes archive file. */
PHP_FUNCTION(archive_file_close)
{
	zval * afile_res = NULL;
	int afile_resid = -1;
	ArchiveFile * afile;
	int retval = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &afile_res) == FAILURE) {
		return;
	}
	ZEND_FETCH_RESOURCE(afile, ArchiveFile *, &afile_res, afile_resid, "archivefile", le_archivefile);

	retval = archive_file_close(afile);
	if (retval != 0) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "archive_file_close: returned error: %d", retval);
	}

	RETURN_LONG(retval);
}
/* }}} archive_file_close */


/* {{{ proto void archive_file_set_error_mode(boolean error_mode)
  Changes error mode of libreadarchive. */
PHP_FUNCTION(archive_file_set_error_mode)
{

	zend_bool error_mode = 0;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &error_mode) == FAILURE) {
		return;
	}

	archive_file_set_error_mode(error_mode ? 1 : 0);
}
/* }}} archive_file_set_error_mode */

#endif /* HAVE_TEXTCONV */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
